/*
 * file CqQueryFolderItem.java
 *
 * Licensed Materials - Property of IBM
 * Restricted Materials of IBM 
 *
 * com.ibm.rational.wvcm.stp.cq.CqQueryFolderItem
 *
 * (C) Copyright IBM Corporation 2006, 2012.  All Rights Reserved.
 * Note to U.S. Government Users Restricted Rights:  Use, duplication or 
 * disclosure restricted by GSA ADP  Schedule Contract with IBM Corp.
 */

package com.ibm.rational.wvcm.stp.cq;

import static com.ibm.rational.wvcm.stpex.StpExBase.PROPERTY_NAMESPACE;

import java.util.List;

import javax.wvcm.Feedback;
import javax.wvcm.ResourceList;
import javax.wvcm.WvcmException;
import javax.wvcm.PropertyNameList.PropertyName;
import javax.wvcm.PropertyRequestItem.PropertyRequest;

import com.ibm.rational.wvcm.stp.StpLocation;
import com.ibm.rational.wvcm.stp.StpResource;

/**
 * A specification for an item in a query folder, which may be a query, chart,
 * report, report format, or a nested query folder.
 * <p>
 * The user-friendly specification for the location of a query folder item has
 * the form
 * 
 * <pre>
 *  <b>cq.query:</b><i>&lt;parent-folder-path&gt;</i>/<i>&lt;item-name&gt;</i>@<i>&lt;db-set&gt;</i>/<i>&lt;user-db&gt;</i>
 * </pre>
 * 
 */
public interface CqQueryFolderItem extends CqContextResource
{
    /** The query folder that contains this folder */
    PropertyName<CqQueryFolder> QUERY_FOLDER =
        new PropertyName<CqQueryFolder>(PROPERTY_NAMESPACE, "query-folder");

    /**
     * Returns the value of the {@link #QUERY_FOLDER QUERY_FOLDER} property as
     * defined by this proxy.
     * 
     * @return A CqQueryFolder proxy for the query folder that contains this
     *         folder. Will be <b>null</b> if there is no containing query
     *         folder.
     * 
     * @throws WvcmException if this proxy does not define a value for the
     *             {@link #QUERY_FOLDER QUERY_FOLDER} property.
     */
    public CqQueryFolder getQueryFolder() throws WvcmException;

    /**
     * The site-extended name for this folder item. 
     */
    PropertyName<StpLocation> SITE_EXTENDED_NAME =
        new PropertyName<StpLocation>(PROPERTY_NAMESPACE, "site-extended-name");

    /**
     * Returns the value of the {@link #SITE_EXTENDED_NAME SITE_EXTENDED_NAME}
     * property as defined by this proxy.
     * 
     * @return An StpLocation containing the site-extended name for this
     *         resource.
     * 
     * @throws WvcmException if this proxy does not define a value for the
     *             {@link #SITE_EXTENDED_NAME SITE_EXTENDED_NAME} property.
     */
    public StpLocation getSiteExtendedName() throws WvcmException;

    /**
     * An indication whether or not a site extended name is required to access
     * this resource unambiguously.
     */
    PropertyName<Boolean> IS_SITE_EXTENDED_NAME_REQUIRED =
        new PropertyName<Boolean>(PROPERTY_NAMESPACE,
                                  "is-site-extended-name-required");

    /**
     * Returns the value of the
     * {@link #IS_SITE_EXTENDED_NAME_REQUIRED IS_SITE_EXTENDED_NAME_REQUIRED}
     * property as defined by this proxy.
     * 
     * @return <b>true</b> if the name of this resource requires a site
     *         extension field to be unambiguous; <b>false</b> if no site
     *         extension field is needed
     * 
     * @throws WvcmException if this proxy does not define a value for the
     *             {@link #IS_SITE_EXTENDED_NAME_REQUIRED IS_SITE_EXTENDED_NAME_REQUIRED}
     *             property.
     */
    public boolean getIsSiteExtendedNameRequired() throws WvcmException;

    /**
     * A list of all the items in the query folder hierarchy whose name differs
     * from the name of this resource only in its site extensions.
     */
    PropertyName<ResourceList<CqQueryFolderItem>> SITE_EXTENDED_NAMES =
        new PropertyName<ResourceList<CqQueryFolderItem>>(PROPERTY_NAMESPACE,
                                                          "site-extended-names");

    /**
     * Returns the value of the {@link #SITE_EXTENDED_NAMES SITE_EXTENDED_NAMES}
     * property as defined by this proxy.
     * 
     * @return A ResourceList containing a proxy for each CqQueryFolderItem
     * resource whose name differs from the name of this resource only in the
     * site extensions used.
     * 
     * @throws WvcmException if this proxy does not define a value for the
     *             {@link #SITE_EXTENDED_NAMES SITE_EXTENDED_NAMES} property.
     */
    public ResourceList<CqQueryFolderItem> getSiteExtendedNames()
        throws WvcmException;

    /**
     * The replica that masters this resource. A resource can be modified only
     * in the replica that masters (has mastership of) it.
     */
    PropertyName<CqReplica> CQ_MASTER_REPLICA =
        new PropertyName<CqReplica>(PROPERTY_NAMESPACE, "cq-master-replica");

    /**
     * Returns the value of the {@link #CQ_MASTER_REPLICA CQ_MASTER_REPLICA}
     * property as defined by this proxy.
     * 
     * @return A CqReplica proxy for the replica that has mastership of this
     *         resource.
     * 
     * @throws WvcmException if this proxy does not define a value for the
     *             {@link #CQ_MASTER_REPLICA CQ_MASTER_REPLICA} property.
     */
    CqReplica getCqMasterReplica() throws WvcmException;

    /**
     * Defines a new value for the {@link #CQ_MASTER_REPLICA CQ_MASTER_REPLICA}
     * property of this proxy.
     * 
     * @param newMasterReplica A CqReplica proxy identifying the replica that
     *            should assume mastership of this resource.
     */
    void setCqMasterReplica(CqReplica newMasterReplica);

    /**
     * Indicates whether or not this query folder item can be updated and/or
     * deleted by the user.
     */
    PropertyName<Boolean> CAN_UPDATE_OR_DELETE =
        new PropertyName<Boolean>(PROPERTY_NAMESPACE, "can-update-or-delete");

    /**
     * Returns the value of the
     * {@link #CAN_UPDATE_OR_DELETE CAN_UPDATE_OR_DELETE} property as defined by
     * this proxy.
     * 
     * @return <b>true</b> if this query folder item can be updated or deleted
     *         by the current user.
     * 
     * @throws WvcmException if this proxy does not define a value for the
     *             {@link #CAN_UPDATE_OR_DELETE CAN_UPDATE_OF_DELETE} property.
     */
    boolean getCanUpdateOrDelete() throws WvcmException;

    /**
     * Makes an editable copy of the query folder item named by this proxy at
     * the location specified in the argument list. Any dirty properties in this
     * proxy are written to the new copy after it is created, <i>not</i> to the
     * resource referenced by this proxy. This item is not modified by this
     * operation.
     * 
     * @param destination The location where a new copy of this query folder
     *            item is to be created. The parent folder of this location must
     *            already exist in the query folder hierarchy of the database.
     * @param overwrite Specifies how a query folder item already at the target
     *            location is to be handled.
     * @param feedback A request for the properties to be defined by the proxy
     *            that is returned by this method.
     * @param deliveryOrder If {@link CqProvider#HOLD}, the copied resource is
     *            left in a writable state in the change context--the copied
     *            resource in the change context must be delivered before it
     *            becomes visible to other providers. If not
     *            {@link CqProvider#HOLD}, the modified and moribund resources
     *            specified by this parameter will be delivered to or deleted
     *            from the database in the order indicated. To deliver all
     *            modified and moribund resources in an arbitrary order, use
     *            {@link CqProvider#DELIVER_ALL}. To deliver just this copied
     *            resource, use {@link CqProvider#DELIVER}. Must not be <b>null</b>.
     * 
     * @return A CqQueryFolderItem proxy for the new copy of this item. Unless
     *         the deliver option was specified, the copied query item will be
     *         editable.
     * 
     * @throws WvcmException if the server is unable to create the copy at the
     *             specified location.
     */
    CqQueryFolderItem doCopy(StpLocation destination,
                             OverwriteMode overwrite,
                             Feedback feedback,
                             List<CqContextResource> deliveryOrder)
        throws WvcmException;

    /**
     * Makes an editable copy of the query folder item named by this proxy at
     * the location specified in the argument list. Any dirty properties are
     * written to the new copy after it is created, <i>not</i> to the resource
     * addressed by this proxy.
     * <p>
     * This variant of doCopy is provided because in the presence of stable or
     * fast location formats the client cannot always guarantee that
     * <code>parentFolder.child(childName)</code> will be a valid location
     * specification.
     * 
     * @param parentFolder The folder to which this query folder item is to be
     *            copied. The proxy must name an existing query folder of this
     *            user database.
     * @param childName The name the copy of this resource is to have in the
     *            destination folder.
     * @param overwrite Specifies how a query folder item already at the target
     *            location is to be handled.
     * @param feedback The query properties to be defined by the proxy that is
     *            returned by this method.
     * @param deliveryOrder Same as the version of
     *            {@link #doCopy(StpLocation, com.ibm.rational.wvcm.stp.cq.CqQueryFolderItem.OverwriteMode, Feedback, List) doCopy taking an StpLocation}.
     * 
     * @return A CqQueryFolderItem proxy for the new copy of this query folder
     *         item. Unless the deliver option was specified, this query
     *         definition will be editable.
     * 
     * @throws WvcmException if the server is unable to create the copy at the
     *             specified location.
     */
    CqQueryFolderItem doCopy(CqQueryFolder parentFolder,
                             String childName,
                             OverwriteMode overwrite,
                             Feedback feedback,
                             List<CqContextResource> deliveryOrder)
        throws WvcmException;

    /**
     * An enumeration of the values for specifying what doCopy and/or doMove
     * should do when a resource already exists at the targeted destination.
     */
    public enum OverwriteMode
    {
        /**
         * If a resource exists at the target location, the operation will
         * fail; the resource at the target location will not be overwritten.
         */
        FORBID,

        /**
         * If a resource exists at the target location, that resource is to be
         * deleted and replaced by the source resource. Specifically, if the
         * target is a folder, that folder and all of its members are deleted
         * and replaced by the source.
         */
        REPLACE,

        /**
         * If the source resource is a folder and a folder exists at the target
         * location, that folder is not deleted, but the contents of the source
         * folder are merged with the contents of the target folder and this
         * rule is applied recursively for each member of the source folder. If
         * either the source resource or the target resource is not a folder,
         * the target resource is deleted and replaced by the source--i.e., in
         * this case, the operation behaves the same as in the REPLACE
         * OverwriteMode.
         */
        MERGE
    }

    /**
     * Makes an editable copy of the query folder item named by this proxy at
     * the location specified in the argument list and deletes the query folder
     * item at the current proxy location. Any dirty properties are written to
     * the new copy after it is created.
     * 
     * @param destination The location to which this query folder item is to be
     *            moved. The location must be in the QUERY namespace of this
     *            user database.
     * @param overwrite Specifies how a query folder item already at the target
     *            location is to be handled.
     * @param feedback The properties to be defined by the proxy that is
     *            returned by this method.
     * @param deliveryOrder Same as the deliveryOrder parameter of
     *            {@link #doCopy(StpLocation, com.ibm.rational.wvcm.stp.cq.CqQueryFolderItem.OverwriteMode, Feedback, List) doCopy}.
     *            But note that use of {@link CqProvider#DELIVER} both delivers
     *            the copy and deletes the original query folder item.
     * 
     * @return A proxy for the new copy of this resource. Unless the deliver
     *         option was specified, the resource at the proxy location will be
     *         editable.
     * 
     * @throws WvcmException if the server is unable to create the copy at the
     *             specified location.
     */
    CqQueryFolderItem doMove(StpLocation destination,
                             OverwriteMode overwrite,
                             Feedback feedback,
                             List<CqContextResource> deliveryOrder)
        throws WvcmException;
    
    /**
     * Directly rename the query item without change any other property of it.
     * <p>
     * 
     * @param parentFolder The folder to which this query folder item is belonged
     *  
     * @param newName The name this resource will be renamed to
     * 
     * @param feedback The properties to be defined by the proxy that is
     *            returned by this method.
     * @param deliveryOrder Same as the deliveryOrder parameter of
     *            {@link #doCopy(StpLocation, com.ibm.rational.wvcm.stp.cq.CqQueryFolderItem.OverwriteMode, Feedback, List) doCopy}.
     * 
     * @return A proxy for the newly renamed resource. Unless the deliver
     *         option was specified, this resource will be editable.
     * 
     * @throws WvcmException if the server is unable to rename the resource with the
     *             new name.
     */
    CqQueryFolderItem doRename(CqQueryFolder parentFolder,
                             String newName,
                             Feedback feedback,
                             List<CqContextResource> deliveryOrder)
        throws WvcmException;


    /**
     * Makes an editable copy of the query folder item named by this proxy in
     * the query folder specified in the argument list and deletes the query
     * folder item at the current proxy location. Any dirty properties are
     * written to the new copy after it is created.
     * <p>
     * This variant of doMove is provided because in the presence of stable or
     * fast location formats the client cannot always guarantee that
     * <code>parentFolder.child(childName)</code> is a valid location
     * specification.
     * 
     * @param parentFolder The folder to which this query folder item is to be
     *            moved. The location must name an existing query folder of this
     *            user database.
     * @param childName The name this resource is to have in the destination
     *            folder.
     * @param overwrite Specifies how a query folder item already at the target
     *            location is to be handled.
     * @param feedback The properties to be defined by the proxy that is
     *            returned by this method.
     * @param deliveryOrder Same as the deliveryOrder parameter of
     *            {@link #doCopy(StpLocation, com.ibm.rational.wvcm.stp.cq.CqQueryFolderItem.OverwriteMode, Feedback, List) doCopy}.
     *            But note that use of {@link CqProvider#DELIVER} both
     *            delivers the copy and deletes the original query folder item.
     * 
     * @return A proxy for the new copy of this resource. Unless the deliver
     *         option was specified, this resource will be editable.
     * 
     * @throws WvcmException if the server is unable to create the copy at the
     *             specified location.
     */
    CqQueryFolderItem doMove(CqQueryFolder parentFolder,
                             String childName,
                             OverwriteMode overwrite,
                             Feedback feedback,
                             List<CqContextResource> deliveryOrder)
        throws WvcmException;
}
